<?php

namespace Crm\School\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Crm\School\Forms\Block as BlockForm;
use Crm\School\Http\Requests\BlockResourceRequest;
use Crm\School\Http\Resources\BlockResource;
use Crm\School\Http\Resources\BlocksCollection;
use Crm\School\Models\Block;
use Crm\School\Actions\BlockAction;
use Crm\School\Actions\BlockActions;


/**
 * Resource controller class for block.
 */
class BlockResourceController extends BaseController
{

    /**
     * Initialize block resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = BlockForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('crm.school.modules'), 'school', guard_url('school'));
            return $next($request);
        });
    }

    /**
     * Display a list of block.
     *
     * @return Response
     */
    public function index(BlockResourceRequest $request)
    {
        $request = $request->all();
        $page = BlockActions::run('paginate', $request);

        $data = new BlocksCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('school::block.names'))
            ->view('school::block.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display block.
     *
     * @param Request $request
     * @param Model   $block
     *
     * @return Response
     */
    public function show(BlockResourceRequest $request, Block $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new BlockResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('school::block.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('school::block.show')
            ->output();
    }

    /**
     * Show the form for creating a new block.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BlockResourceRequest $request, Block $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new BlockResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('school::block.name'))
            ->view('school::block.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new block.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BlockResourceRequest $request, Block $model)
    {
        try {
            $request = $request->all();
            $model = BlockAction::run('store', $model, $request);
            $data = new BlockResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('school::block.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('school/block/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/school/block'))
                ->redirect();
        }

    }

    /**
     * Show block for editing.
     *
     * @param Request $request
     * @param Model   $block
     *
     * @return Response
     */
    public function edit(BlockResourceRequest $request, Block $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new BlockResource($model);
        // return view('school::block.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('school::block.name'))
            ->view('school::block.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the block.
     *
     * @param Request $request
     * @param Model   $block
     *
     * @return Response
     */
    public function update(BlockResourceRequest $request, Block $model)
    {
        try {
            $request = $request->all();
            $model = BlockAction::run('update', $model, $request);
            $data = new BlockResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('school::block.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('school/block/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('school/block/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the block.
     *
     * @param Model   $block
     *
     * @return Response
     */
    public function destroy(BlockResourceRequest $request, Block $model)
    {
        try {

            $request = $request->all();
            $model = BlockAction::run('destroy', $model, $request);
            $data = new BlockResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('school::block.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('school/block/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('school/block/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
